/*
 * JFBTERM for FreeBSD
 * Copyright (C) 2008 Yusuke BABA <babayaga1@y8.dion.ne.jp>
 *
 * JFBTERM/ME - J framebuffer terminal/Multilingual Enhancement -
 * Copyright (C) 2003 Fumitoshi UKAI <ukai@debian.or.jp>
 * Copyright (C) 1999 Noritoshi MASUICHI <nmasu@ma3.justnet.ne.jp>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY YUSUKE BABA ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL YUSUKE BABA BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef INCLUDE_FONT_H
#define INCLUDE_FONT_H

#include <sys/types.h>
#include <stdbool.h>
#include <stdio.h>

#include "getcap.h"

#define TFONT_FT_SINGLE  (0x00000000)           /* 1 byte character set */
#define TFONT_FT_DOUBLE  (0x01000000)           /* 2 byte character set */
#define TFONT_FT_94CHAR  (0x00000000)           /* 94 or 94^n */
#define TFONT_FT_96CHAR  (0x02000000)           /* 96 or 96^n */
#define TFONT_FT_OTHER   (0x10000000)           /* other coding system */

typedef enum {
	FH_LEFT,
	FH_RIGHT,
	FH_UNI
} FONTSET_HALF;

typedef struct Raw_TFont {
	const u_char *(*conv)(struct Raw_TFont *p, u_short c, u_int * width);
	/* --- */
	const char *name;
	u_int width;            /* ʸοʿɥåȿ */
	u_int height;           /* ʸοľɥåȿ */
	/* */
	u_int fsignature;
	FONTSET_HALF fhalf;
	bool alias;
	/* */
	u_char **glyph;         /* ӥåȥޥåγglyph Ƭ */
	u_int *glyph_width;
	u_char *dglyph;         /* бglyph¸ߤʤglyph */
	u_char *bitmap;         /* ӥåȥޥå */
	u_int colf;
	u_int coll;
	u_int rowf;
	u_int rowl;
	u_int colspan;          /* = coll-colf+1; */
	u_int bytew;            /* ʸοʿ饤ΥХȿ */
	u_int bytec;            /* ʸΥХȿ */
} TFont;

extern TFont gFont[];
extern u_int gFontsWidth;
extern u_int gFontsHeight;

void font_initialize(void);
void font_configure(TCaps *caps);
void font_showList(FILE *stream);
int font_getIndexBySignature(const u_int fsignature);
int font_getIndexByName(const char *name);
bool font_isLoaded(TFont *p);
void font_draw(TFont *p, u_short chlw, u_char fcol, u_char bcol, u_int x, u_int y, u_int w);

#endif /* INCLUDE_FONT_H */

