/*
 * JFBTERM for FreeBSD
 * Copyright (C) 2008 Yusuke BABA <babayaga1@y8.dion.ne.jp>
 *
 * JFBTERM Unofficial eXtension Patch
 * Copyright (C) 2006 Young Chol Song <nskystars at yahoo dot com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY YUSUKE BABA ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL YUSUKE BABA BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_SPLASH_SCREEN

#include <sys/types.h>
#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "read-bmp.h"

u_char *read_bmp_file(FILE *stream, int *width, int *height)
{
	struct bmp_file *bmp;
	u_int data_size;
	int x, y, line_len;
	int tbuf;
	u_char *data;

	bmp = (struct bmp_file *)calloc(1, sizeof(struct bmp_file));
	if (bmp == NULL) {
		warnx("Bitmap file read canceled: %s", strerror(errno));
		return NULL;
	}
	if ((fgetc(stream) | (fgetc(stream) << 8)) != BMP_SIGNATURE) {
		warnx("Bitmap file format error: Bad signature.");
		free(bmp);
		return NULL;
	}
	fread(&(bmp->hdr), sizeof(struct bmp_hdr), 1, stream);
	if (bmp->hdr.compression != 0) {
		warnx("Bitmap file format error: Compressed bitmap.");
		free(bmp);
		return NULL;
	}
	*width = bmp->hdr.width;
	*height = bmp->hdr.height;
	line_len = *width * 3;
	/* 32 bit boundary for each line */
	if (line_len % 4)
		line_len = line_len + (4 - (line_len % 4));
	if (bmp->hdr.bit_count == 24)
		data_size = line_len * *height;
	else {
		warnx("Bitmap file format error: Not 24 bit.");
		free(bmp);
		return NULL;
	}
	fseek(stream, bmp->hdr.offset, SEEK_SET);
	bmp->data = (u_char *)malloc(data_size * sizeof(u_char));
	if (bmp->data == NULL) {
		warnx("Bitmap file read canceled: %s", strerror(errno));
		free(bmp);
		return NULL;
	}
	fread(bmp->data, 1, data_size, stream);
	data = (u_char *)malloc((*width * 3) * *height * sizeof(u_char));
	if (data == NULL) {
		warnx("Bitmap file read canceled: %s", strerror(errno));
		free(bmp->data);
		free(bmp);
		return NULL;
	}
	for (y = 0; y < *height; y++)
		memcpy(data + y * (*width * 3),
		       bmp->data + (*height - y - 1) * line_len,
		       (*width * 3));
	for (y = 0; y < *height; y++) {
		for (x = 0; x < (*width * 3); x += 3) {
			tbuf = data[y * (*width * 3) + x];
			data[y * (*width * 3) + x] = data[y * (*width * 3) + x + 2];
			data[y * (*width * 3) + x + 2] = tbuf;
		}
	}
	free(bmp->data);
	free(bmp);
	return data;
}

#endif /* ENABLE_SPLASH_SCREEN */

