/*
 * JFBTERM for FreeBSD
 * Copyright (C) 2008 Yusuke BABA <babayaga1@y8.dion.ne.jp>
 *
 * KON2 for FreeBSD
 * Copyright (C) Takashi OGURA
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY YUSUKE BABA ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL YUSUKE BABA BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <errno.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>

#include "utilities.h"

int lookup(const char *s, ...)
{
	va_list va;
	char *p;
	int i, result;

	result = -1;
	if (s != NULL) {
		va_start(va, s);
		for (i = 0; ((p = va_arg(va, char *)) != NULL); i++) {
			if (strcmp(s, p) == 0) {
				result = i;
				break;
			}
		}
		va_end(va);
	}
	return result;
}

int strrcmp(const char *s1, const char *s2)
{
	return strcmp(s1 + strlen(s1) - strlen(s2), s2);
}

int strrep(char *p, const int len, const int maxlen, const int oldlen, const char *newstr)
{
	int newlen, copylen, diff;

	newlen = strlen(newstr);
	diff = newlen - oldlen;
	copylen = len - oldlen;
	if (len + diff > maxlen) {
		diff -= (len + diff - maxlen);
		copylen = maxlen - newlen;
	}
	if (copylen > 0)
		memmove(p + newlen, p + oldlen, copylen);
	copylen = newlen;
	if (newlen > maxlen) {
		copylen = maxlen;
		diff = maxlen - len;
	}
	if (copylen > 0)
		memcpy(p, newstr, copylen);
	return diff;
}

/* Same as digittoint(3) */
int hex2int(const char c)
{
	int result;

	result = 0;
	if (c >= '0' && c <= '9')
		result = c - '0';
	else if (c >= 'a' && c <= 'f')
		result = c - 'a' + 10;
	else if (c >= 'A' && c <= 'F')
		result = c - 'A' + 10;
	return result;
}

ssize_t write_wrapper(int fd, const void *buf, size_t nbytes)
{
	ssize_t n, total;

	total = 0;
	while (total < nbytes) {
		n = write(fd, buf, nbytes - total);
		if (n == -1) n = 0;
		total += n;
		if (errno != EINTR)
			break;
	}
	return total;
}

